import java.util.stream.*;
import java.awt.Color;

import graph.*;

public class Client {
    public static final int TRIALS = 25;    // Total number of trials ran per n value
    public static final int WARMUP = 23;    // How many iterations of n to skip over while
                                            // the system warms up
    
    public static final int MIN_VALUES = 1000;  // Min n
    public static final int MAX_VALUES = 10000; // Max n
    public static final int INCR = 100;         // How much to increment n by

    public static void main(String[] args) {
        ColorGraph g = new MultiGraph();
        IntList[] lists = { new ArrayIntList(MAX_VALUES + 1), new LinkedIntList() };
        Color[] colors = { Color.RED, Color.BLUE };

        for (int i = 0; i < lists.length; i++) {
            IntList list = lists[i];
            Line l = new Line();
            for (int n = MIN_VALUES; n <= MAX_VALUES; n += INCR) {
                double val = singleRound(list, n);

                if (n > MIN_VALUES + WARMUP * INCR) {
                    double newN = n - MIN_VALUES - WARMUP * INCR;
                    l.addPoint(new Point(newN / INCR, val / 1000));
                }
            }
            g.add(l, colors[i]);
        }
        g.save("test.png");
    }

    // Runs a single round of trials on the current operation of a list storing 'max' values.
    // Averages out the results to account for outliers
    public static double singleRound(IntList list, int max) {
        long total = 0;
        for (int i = 0; i < TRIALS; i++) {
            list.copy(IntStream.rangeClosed(1, max)
                               .boxed()
                               .collect(Collectors.toList()));

            long start = System.nanoTime();

            // TODO: edit the operation to be tested
            list.size();
            
            long end = System.nanoTime();

            total += end - start;
        }
        return (double)(total) / (TRIALS);
    }
}
