import java.util.*;

// Class LinkedIntList can be used to store a list of integers.

public class LinkedIntListSize implements IntList {
    private ListNode front;
    private int size;

    // post: returns the total number of elements within this list
    public int size() {
        return size;
    }

    // pre: throws an IndexOutOfBoundsException if the provided index is invalid
    //      (index < 0 || index >= size)
    // post: returns the value at the given index of the list.
    public int get(int index) {
        if (index < 0 || index >= size()) {
            throw new IndexOutOfBoundsException("Provided index invalid: " + index);
        }
        
        ListNode curr = front;
        for (int i = 0; i < index; i++) {
            curr = curr.next;
        }
        return curr.data;
    }

    // post: returns a comma-separated string representation of the list surrounded
    //       by square brackets
    public String toString() {
        if (front == null) {
            return "[]";
        }
        
        String ret = "[" + front.data;
        ListNode curr = front.next;
        while (curr != null) {
            ret += ", " + curr.data;
            curr = curr.next;
        }
        return ret + "]";
    }
    
    // post: returns the index of first occurence of the provided value within the
    //       list, -1 if not present
    public int indexOf(int value) {
        int index = 0;
        ListNode curr = front;
        while (curr != null) {
            if (curr.data == value) {
                return index;
            }
            index++;
            curr = curr.next;
        }
        return -1;
    }
    
    // post: returns whether or not the given value is present within the list
    public boolean contains(int value) {
        return indexOf(value) != -1;
    }

    // post: adds the given value to the end of the list
    public void add(int value) {
        add(size(), value);
    }
    
    // pre: throws an IndexOutOfBoundsException if the provided index is invalid
    //      (index < 0 || index > size)
    // post: Adds the given value at the given index
    public void add(int index, int value) {
        if (index < 0 || index > size()) {
            throw new IndexOutOfBoundsException("Provided index invalid: " + index);
        }

        if (index == 0) {
            this.front = new ListNode(value, this.front);
        } else {
            ListNode curr = front;
            for (int i = 0; i < index - 1; i++) {
                curr = curr.next;
            }
            curr.next = new ListNode(value, curr.next);
        }
        size++;
    }
    
    // pre: throws an IndexOutOfBoundsException if the provided index is invalid
    //      (index < 0 || index >= size)
    // post: removes the value at the given index
    public void remove(int index) {
        if (index < 0 || index >= size()) {
            throw new IndexOutOfBoundsException("Provided index invalid: " + index);
        }

        if (index == 0) {
            this.front = this.front.next;
        } else {
            ListNode curr = front;
            for (int i = 0; i < index - 1; i++) {
                curr = curr.next;
            }
            curr.next = curr.next.next;
        }
        size--;
    }

    // Adds all the values from the provided list into this one
    public void copy(List<Integer> other) {
        this.front = null;
        for (int i = other.size() - 1; i >= 0; i--) {
            this.front = new ListNode(other.get(i), this.front);
        }
        size = other.size();
    }

    // Private inner class capable of storing individual data values
    private static class ListNode {
        public final int data;
        public ListNode next;

        // Constructs a new ListNode storing the given data value
        public ListNode(int data) {
            this(data, null);
        }

        // Constructs a new ListNode storing the given data value and reference
        // to next node in the chain
        public ListNode(int data, ListNode next) {
            this.data = data;
            this.next = next;
        }
    }
}
