package graph;

import java.awt.Color;
import java.util.*;

// Represents a mathmatical graph, displaying points and lines added by a user
public class MonoGraph extends Graph {
    private Set<Graphable> elements;
    private Color color;

    // Behavior: Creates a new, empty graph displaying 2d space from
    //           [-10, 10] on each axis with default color Color.RED
    public MonoGraph() {
        this(Color.RED);
    }

    // Behavior: Creates a new, empty graph displaying 2d space from
    //           [-10, 10] on each axis where points will be colored
    //           'color'
    // Parameters:
    //      color - The color used by the graph.
    public MonoGraph(Color color) {
        this.elements = new HashSet<>();
        this.color = color;
    }

    // Behavior: Adds the given graphable element 'g' to be displayed within this graph
    // Parameters:
    //      g - The element to be added
    @Override
    public void add(Graphable g) {
        this.elements.add(g);
    }

    // Behavior: Returns the color of the provided graphable 'g', null if not present
    @Override
    public Color getColor(Graphable g) {
        if (!this.elements.contains(g)) {
            return null;
        }
        return this.color;
    }

    // Behavior: Returns a set of all the elements present in this graph
    @Override
    public Set<Graphable> getElements() {
        return this.elements;
    }
}
